@@ -23,6 +23,7 @@ ways, as listed at the top of each
 
 */
 
+#include <_G_config.h>
 #include "Int.h"
 #include "Char.h"
 
@@ -300,7 +301,7 @@ ways, as listed at the top of each
 #undef	NOSTRUCTASSIGN		/* Define if compiler can't assign structures */
 
 /* define only one of the next two defines */
-#ifndef USG
+#ifndef _G_SYSV
 #define TIMES			/* Use times(2) time function */
 #else
 #define TIME			/* Use time(2) time function */
@@ -310,6 +311,9 @@ ways, as listed at the top of each
 #include <sys/types.h>
 #include <sys/times.h>
 #endif
+#ifdef TIME
+#include <time.h>
+#endif
 
 /* define the granularity of your times(2) function (when used) */
 #ifndef HZ
@@ -436,15 +440,14 @@ void Proc0()
 	String30		String2Loc;
 
 #ifdef TIME
-	long			time();
 	long			starttime;
 	long			benchtime;
 	long			nulltime;
 	register unsigned int	i;
 
-	starttime = time( (long *) 0);
+	starttime = time( (_G_time_t *) 0);
 	for (i = 0; i < LOOPS; ++i);
-	nulltime = time( (long *) 0) - starttime; /* Computes o'head of loop */
+	nulltime = time( (_G_time_t*) 0) - starttime; /* Computes o'head of loop */
 #endif
 #ifdef TIMES
 	time_t			starttime;
@@ -472,7 +475,7 @@ void Proc0()
 -- Start Timer --
 *****************/
 #ifdef TIME
-	starttime = time( (long *) 0);
+	starttime = time( (_G_time_t*) 0);
 #endif
 #ifdef TIMES
 	times(&Tms); starttime = Tms.tms_utime;
@@ -509,7 +512,7 @@ void Proc0()
 *****************/
 
 #ifdef TIME
-	benchtime = time( (long *) 0) - starttime - nulltime;
+	benchtime = time( (_G_time_t *) 0) - starttime - nulltime;
 	printf("Dhrystone time for %ld passes = %ld\n",
 		(long) LOOPS, benchtime);
 	printf("This machine benchmarks at %ld dhrystones/second\n",
@@ -570,7 +573,7 @@ void Proc2(OneToFifty	*IntParIO)
 
 void Proc3(RecordPtr	*PtrParOut)
 {
-	if (PtrGlb != NULL)
+	if (PtrGlb)
 		*PtrParOut = PtrGlb->PtrComp;
 	else
 		IntGlob = 100;
