@@ -1,5 +1,13 @@
 #ifndef __libgxx_dirent_h
 
+#include <_G_config.h>
+
+#if !_G_HAVE_DIRENT
+#define __libgxx_dirent_h
+#define direct dirent
+#include <sys/dir.h>
+#else
+
 extern "C" {
 
 #ifdef __dirent_h_recursive
@@ -11,9 +19,6 @@ extern "C" {
 #define readdir __hide_readdir
 #define telldir __hide_telldir
 #define seekdir __hide_seekdir
-#if ! (defined(__ultrix__) || defined(__sun__))
-#define rewinddir __hide_rewinddir
-#endif
 
 #include_next <dirent.h>
 
@@ -23,19 +28,16 @@ extern "C" {
 #undef readdir
 #undef telldir
 #undef seekdir
-#if ! (defined(__ultrix__) || defined(__sun__))
-#undef rewinddir
-#endif
 
 DIR *opendir(const char *);
 int closedir(DIR *);
 struct dirent *readdir(DIR *);
 long telldir(DIR *);
 void seekdir(DIR *, long);
-#ifndef rewinddir
-void rewinddir(DIR *);
-#endif
+// We don't bother with rewinddir (many systems define it as a macro).
+// void rewinddir(DIR *);
 #endif
 }
 
 #endif
+#endif
