@@ -2,8 +2,15 @@
 #ifndef _string_h
 #define _string_h 1
 
-#include <stddef.h>
+#include <_G_config.h>
 
+#ifndef size_t
+#define size_t _G_size_t
+#endif
+
+#ifndef NULL
+#define NULL _G_NULL
+#endif
 
 extern "C" {
 
@@ -25,21 +32,10 @@ char*     strstr(const char*, const char *);
 char*     strtok(char*, const char*);
 size_t    strxfrm(char*, const char*, size_t);
 
-}
-
-#ifdef USG
-inline char* index(const char* s, int c) { return strchr(s, c); }
-inline char* rindex(const char* s, int c) { return strrchr(s, c); }
-
-#else /* not USG */
-
-extern "C" {
 char*     index(const char*, int);
 char*     rindex(const char*, int);
 }
 
-#endif
-
 #include <memory.h>
 
 #endif
