@@ -20,16 +20,18 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 /* But note that some systems that use sys/signal.h to define signal.h. */
 
 #ifndef __libgxx_sys_signal_h
-
-extern "C" {
 #ifdef __sys_signal_h_recursive
 #include_next <sys/signal.h>
 #else
 #define __sys_signal_h_recursive
+
+extern "C" {
+#define signal __hide_signal
 #include_next <sys/signal.h>
-#define __libgxx_sys_signal_h 1
-#endif
+#undef signal
 }
 
+#define __libgxx_sys_signal_h 1
+#endif
 #endif
 
