@@ -29,6 +29,7 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <AllocRing.h>
 #include <new.h>
 #include <builtin.h>
+#include <strstream.h>
 
 void BitString::error(const char* msg) const
 {
@@ -1970,32 +1971,39 @@ BitPattern atoBitPattern(const char* s, char f,char t,char x)
 
 extern AllocRing _libgxx_fmtq;
 
-const char* BitStringtoa(const BitString& x, char f, char t)
+void BitString::printon(ostream& os, char f, char t) const
 {
-  int wrksiz = x.length() + 2;
-  char* fmtbase = (char *) _libgxx_fmtq.alloc(wrksiz);
-  char* fmt = fmtbase;
-  unsigned int  xl = x.rep->len;
-  const unsigned short* s = x.rep->s;
+  int wrksiz = length() + 2;
+  unsigned int  xl = rep->len;
+  const unsigned short* ptr = rep->s;
+  register streambuf *sb = os.rdbuf();
   unsigned short a = 0;
 
   for (unsigned int  i = 0; i < xl; ++i)
   {
     if (i % BITSTRBITS == 0)
-      a = *s++;
-    *fmt++ = (a & 1)? t : f;
+      a = *ptr++;
+    sb->sputc((a & 1)? t : f);
     a >>= 1;
   }
-
-  *fmt = 0;
-
+}
+const char* BitStringtoa(const BitString& x, char f, char t)
+{
+  int wrksiz = x.length() + 2;
+  char* fmtbase = (char *) _libgxx_fmtq.alloc(wrksiz);
+  char* fmt = fmtbase;
+  ostrstream stream(fmtbase, wrksiz);
+  
+  x.printon(stream, f, t);
+  stream << ends;
   return fmtbase;
 }
 
-
 ostream& operator << (ostream& s, const BitString& x)
 {
-  return s << BitStringtoa(x);
+  if (s.opfx())
+    x.printon(s);
+  return s;
 }
 
 const char* BitPatterntoa(const BitPattern& p, char f,char t,char x)
@@ -2006,10 +2014,22 @@ const char* BitPatterntoa(const BitPattern& p, char f,char t,char x)
 
   int wrksiz = l + 2;
   char* fmtbase = (char *) _libgxx_fmtq.alloc(wrksiz);
-  char* fmt = fmtbase;
+  ostrstream stream(fmtbase, wrksiz);
+  
+  p.printon(stream, f, t, x);
+  stream << ends;
+  return fmtbase;
+}
 
-  const unsigned short* ps = p.pattern.rep->s;
-  const unsigned short* ms = p.mask.rep->s;
+void BitPattern::printon(ostream& s, char f,char t,char x) const
+{
+  unsigned int  pl = pattern.rep->len;
+  unsigned int  ml = mask.rep->len;
+  unsigned int  l = (pl <= ml)? pl : ml;
+  register streambuf *sb = s.rdbuf();
+
+  const unsigned short* ps = pattern.rep->s;
+  const unsigned short* ms = mask.rep->s;
   unsigned short a = 0;
   unsigned short m = 0;
 
@@ -2021,21 +2041,19 @@ const char* BitPatterntoa(const BitPattern& p, char f,char t,char x)
       m = *ms++;
     }
     if (m & 1)
-      *fmt++ =(a & 1)? t : f;
+      sb->sputc((a & 1)? t : f);
     else
-      *fmt++ = x;
+      sb->sputc(x);
     a >>= 1;
     m >>= 1;
   }
-
-  *fmt = 0;
-  return fmtbase;
 }
 
-
 ostream& operator << (ostream& s, const BitPattern& x)
 {
-  return s << BitPatterntoa(x);
+  if (s.opfx())
+    x.printon(s);
+  return s;
 }
 
 
