@@ -66,6 +66,10 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #define MALLOC_MIN_OVERHEAD 4
 #endif
 
+IntRep _ZeroRep = {1, 0, 1, {0}};
+IntRep _OneRep = {1, 0, 1, {1}};
+IntRep _MinusOneRep = {1, 0, 0, {1}};
+
 
 // utilities to extract and transfer bits 
 
@@ -177,7 +181,7 @@ IntRep* Ialloc(IntRep* old, const unsigned short* src, int srclen, int newsgn,
   scpy(src, rep->s, srclen);
   Iclear_from(rep, srclen);
 
-  if (old != rep && old != 0) delete old;
+  if (old != rep && old != 0 && !STATIC_IntRep(old)) delete old;
   return rep;
 }
 
@@ -188,7 +192,7 @@ IntRep* Icalloc(IntRep* old, int newlen)
   IntRep* rep;
   if (old == 0 || newlen > old->sz)
   {
-    if (old != 0) delete old;
+    if (old != 0 && !STATIC_IntRep(old)) delete old;
     rep = Inew(newlen);
   }
   else
@@ -221,7 +225,7 @@ IntRep* Iresize(IntRep* old, int newlen)
       rep = Inew(newlen);
       scpy(old->s, rep->s, oldlen);
       rep->sgn = old->sgn;
-      delete old;
+      if (!STATIC_IntRep(old)) delete old;
     }
     else
       rep = old;
@@ -257,7 +261,7 @@ IntRep* Icopy(IntRep* old, const IntRep* src)
     int newlen = src->len;
     if (old == 0 || newlen > old->sz)
     {
-      if (old != 0) delete old;
+      if (old != 0 && !STATIC_IntRep(old)) delete old;
       rep = Inew(newlen);
     }
     else
@@ -294,7 +298,7 @@ IntRep* Icopy_long(IntRep* old, long x)
   IntRep* rep;
   if (old == 0 || srclen > old->sz)
   {
-    if (old != 0) delete old;
+    if (old != 0 && !STATIC_IntRep(old)) delete old;
     rep = Inew(srclen);
   }
   else
@@ -313,36 +317,30 @@ IntRep* Icopy_long(IntRep* old, long x)
 
 IntRep* Icopy_zero(IntRep* old)
 {
-  IntRep* rep;
   if (old == 0)
-    rep = Inew(0);
-  else
-    rep = old;
+    return &_ZeroRep;
 
-  rep->len = 0;
-  rep->sgn = I_POSITIVE;
+  old->len = 0;
+  old->sgn = I_POSITIVE;
 
-  return rep;
+  return old;
 }
 
 // special case for 1 or -1
 
 IntRep* Icopy_one(IntRep* old, int newsgn)
 {
-  IntRep* rep;
   if (old == 0 || 1 > old->sz)
   {
-    if (old != 0) delete old;
-    rep = Inew(1);
+    if (old != 0 && !STATIC_IntRep(old)) delete old;
+    return newsgn==I_NEGATIVE ? &_MinusOneRep : &_OneRep;
   }
-  else
-    rep = old;
 
-  rep->sgn = newsgn;
-  rep->len = 1;
-  rep->s[0] = 1;
+  old->sgn = newsgn;
+  old->len = 1;
+  old->s[0] = 1;
 
-  return rep;
+  return old;
 }
 
 // convert to a legal two's complement long if possible
@@ -1234,8 +1232,8 @@ IntRep* div(const IntRep* x, const IntRep* y, IntRep* q)
     q = Icalloc(q, ql);
     do_divide(r->s, yy->s, yl, q->s, ql);
 
-    if (yy != y) delete yy;
-    delete r;
+    if (yy != y && !STATIC_IntRep(yy)) delete yy;
+    if (!STATIC_IntRep(r)) delete r;
   }
   q->sgn = samesign;
   Icheck(q);
@@ -1302,7 +1300,7 @@ IntRep* div(const IntRep* x, long y, IntRep* q)
     q = Icalloc(q, ql);
     do_divide(r->s, ys, yl, q->s, ql);
 
-    delete r;
+    if (!STATIC_IntRep(r)) delete r;
   }
   q->sgn = samesign;
   Icheck(q);
@@ -1384,7 +1382,7 @@ void divide(const Integer& Ix, long y, Integer& Iq, long& rem)
     }
     Icheck(r);
     rem = Itolong(r);
-    delete r;
+    if (!STATIC_IntRep(r)) delete r;
   }
   rem = abs(rem);
   if (xsgn == I_NEGATIVE) rem = -rem;
@@ -1453,7 +1451,7 @@ void divide(const Integer& Ix, const Integer& Iy, Integer& Iq, Integer& Ir)
     q = Icalloc(q, ql);
     do_divide(r->s, yy->s, yl, q->s, ql);
 
-    if (yy != y) delete yy;
+    if (yy != y && !STATIC_IntRep(yy)) delete yy;
     if (prescale != 1)
     {
       Icheck(r);
@@ -1507,7 +1505,7 @@ IntRep* mod(const IntRep* x, const IntRep* y, IntRep* r)
       
     do_divide(r->s, yy->s, yl, 0, xl - yl + 1);
 
-    if (yy != y) delete yy;
+    if (yy != y && !STATIC_IntRep(yy)) delete yy;
 
     if (prescale != 1)
     {
@@ -1947,8 +1945,8 @@ IntRep* gcd(const IntRep* x, const IntRep* y)
       t = add(t, 0, u, 0, t);
     }
   }
-  delete t;
-  delete v;
+  if (!STATIC_IntRep(t)) delete t;
+  if (!STATIC_IntRep(v)) delete v;
   if (k != 0) u = lshift(u, k, u);
   return u;
 }
@@ -2006,7 +2004,7 @@ IntRep* power(const IntRep* x, long y, IntRep* r)
       else
         b = multiply(b, b, b);
     }
-    delete b;
+    if (!STATIC_IntRep(b)) delete b;
   }
   r->sgn = sgn;
   Icheck(r);
@@ -2160,24 +2158,31 @@ ostream& operator << (ostream& s, const Integer& y)
 #ifdef _OLD_STREAMS
   return s << Itoa(y.rep);
 #else
-  int base = (s.flags() & ios::oct) ? 8 : (s.flags() & ios::hex) ? 16 : 10;
+  if (s.opfx())
+    {
+      int base = (s.flags() & ios::oct) ? 8 : (s.flags() & ios::hex) ? 16 : 10;
+      int width = s.width();
+      y.printon(s, base, width);
+    }
+  return s;
+#endif
+}
+
+void Integer::printon(ostream& s, int base /* =10 */, int width /* =0 */) const
+{
   int align_right = !(s.flags() & ios::left);
   int showpos = s.flags() & ios::showpos;
   int showbase = s.flags() & ios::showbase;
   char fillchar = s.fill();
   char Xcase = (s.flags() & ios::uppercase)? 'X' : 'x';
-  int width = s.width();
-  const IntRep* x = y.rep;
+  const IntRep* x = rep;
   int fmtlen = (x->len + 1) * I_SHIFT / lg(base) + 4 + width;
   char* fmtbase = new char[fmtlen];
   char* f = cvtItoa(x, fmtbase, fmtlen, base, showbase, width, align_right,
-                    fillchar, Xcase, showpos);
+		    fillchar, Xcase, showpos);
   s.write(f, fmtlen);
   delete fmtbase;
-  return s;
-#endif
 }
-  
 
 char*  cvtItoa(const IntRep* x, char* fmt, int& fmtlen, int base, int showbase,
               int width, int align_right, char fillchar, char Xcase, 
@@ -2222,7 +2227,7 @@ char*  cvtItoa(const IntRep* x, char* fmt, int& fmtlen, int base, int showbase,
             ch += '0';
           *--s = ch;
         }
-        delete z;
+	if (!STATIC_IntRep(z)) delete z;
         break;
       }
       else
@@ -2390,16 +2395,20 @@ istream& operator >> (istream& s, Integer& y)
 
 int Integer::OK() const
 {
-  int v = rep != 0;             // have a rep
-  int l = rep->len;
-  int s = rep->sgn;
-  v &= l <= rep->sz;            // length with bounds
-  v &= s == 0 || s == 1;        // legal sign
-  Icheck(rep);                  // and correctly adjusted
-  v &= rep->len == l;
-  v &= rep->sgn == s;
-  if (!v) error("invariant failure");
-  return v;
+  if (rep != 0)
+    {
+      int l = rep->len;
+      int s = rep->sgn;
+      int v = l <= rep->sz || STATIC_IntRep(rep);    // length within bounds
+      v &= s == 0 || s == 1;        // legal sign
+      Icheck(rep);                  // and correctly adjusted
+      v &= rep->len == l;
+      v &= rep->sgn == s;
+      if (v)
+	  return v;
+    }
+  error("invariant failure");
+  return 0;
 }
 
 void Integer::error(const char* msg) const
