@@ -1206,16 +1206,17 @@ istream& operator>>(istream& s, String& x)
     return s;
   }
 #else
-  if (!s.ipfx(0))
+  if (!s.ipfx(0) || (!(s.flags() & ios::skipws) && !ws(s)))
   {
     s.set(ios::failbit); // Redundant if using GNU iostreams.
     return s;
   }
 #endif
-  char ch;
+  int ch;
   int i = 0;
   x.rep = Sresize(x.rep, 20);
-  while (s.get(ch))
+  register streambuf *sb = s.rdbuf();
+  while ((ch = sb->sbumpc()) != EOF)
   {
     if (isspace(ch))
       break;
@@ -1225,7 +1226,8 @@ istream& operator>>(istream& s, String& x)
   }
   x.rep->s[i] = 0;
   x.rep->len = i;
-  if(i == 0) s.clear(_bad);
+  if (i == 0) s.set(_fail);
+  if (ch == EOF) s.set(_eof);
   return s;
 }
 
@@ -1239,10 +1241,11 @@ int readline(istream& s, String& x, char terminator, int discard)
   {
     return 0;
   }
-  char ch;
+  int ch;
   int i = 0;
   x.rep = Sresize(x.rep, 80);
-  while (s.get(ch))
+  register streambuf *sb = s.rdbuf();
+  while ((ch = sb->sbumpc()) != EOF)
   {
     if (ch != terminator || !discard)
     {
@@ -1255,6 +1258,7 @@ int readline(istream& s, String& x, char terminator, int discard)
   }
   x.rep->s[i] = 0;
   x.rep->len = i;
+  if (ch == EOF) s.set(_eof);
   return i;
 }
 
@@ -1305,11 +1309,11 @@ int String::freq(char c) const
 
 int String::OK() const
 {
-  int v = rep != 0;             // have a rep
-  v &= rep->len <= rep->sz;     // string within bounds
-  v &= rep->s[rep->len] == 0;   // null-terminated
-  if (!v) error("invariant failure");
-  return v;
+  if (rep == 0             // don't have a rep
+    || rep->len > rep->sz     // string oustide bounds
+    || rep->s[rep->len] != 0)   // not null-terminated
+      error("invariant failure");
+  return 1;
 }
 
 int SubString::OK() const
