@@ -1,5 +1,3 @@
-#ifndef NO_LIBGXX_MALLOC
-
 // This may look like C code, but it is really -*- C++ -*-
 
 /* 
@@ -22,13 +20,12 @@ Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 #ifdef __GNUG__
 #pragma implementation
 #endif
-#include <stddef.h>
-#include <stdlib.h>
+#include <new.h>
+
+#if 0 /* ndef NO_LIBGXX_MALLOC */
 
 void* operator new(size_t n) 
 {
   return malloc (n);
 }
-
-
 #endif
