@@ -1,9 +1,10 @@
 // Use all the g++ headerfiles
 
 // $Author: bothner $
-// $Revision: 1.6 $
-// $Date: 1992/03/11 02:19:29 $
+// $Revision: 1.9 $
+// $Date: 1992/05/14 22:09:28 $
 
+#include <_G_config.h>
 // If we have the old iostream library, it defines _OLD_STREAMS
 #include <stream.h>
 
@@ -11,13 +12,17 @@
 
 #include <sys/types.h>
 #include <sys/time.h>
+#if _G_HAVE_SYS_RESOURCE
 #include <sys/resource.h>
+#endif
 #include <sys/file.h>
 #include <fcntl.h>
 #include <sys/param.h>
 #include <sys/wait.h>
 #include <sys/signal.h>
+#if _G_HAVE_SYS_SOCKET
 #include <sys/socket.h>
+#endif
 #include <sys/stat.h>
 #include <sys/times.h>
 
@@ -67,7 +72,6 @@
 #include <Weibull.h>
 
 #include <assert.h>
-#include <bool.h>
 #include <bstring.h>
 #include <libc.h>
 #include <compare.h>
